///////////////////////////////////////////////////////////////////////////////
//
//  GPIB488.H
//
// Copyright 2000, 2003 Interface Corpration. All rights reserved.
//
///////////////////////////////////////////////////////////////////////////////
#if !defined( GPIB488_H_ )
#define GPIB488_H_

#ifdef __cplusplus
extern "C" {
#endif

// -----------------------------------------------------------------------
//	Constans
// -----------------------------------------------------------------------
// Multiline Interface message
#define UNL  0x3f	// Unlisten
#define UNT  0x5f	// Untalk
#define GTL  0x01	// Go To Local
#define SDC  0x04	// Selected Device Clear
#define PPC  0x05	// Parallel Poll Configure
#define GET  0x08	// Group Execute Trigger
#define TCT  0x09	// Take Control
#define LLO  0x11	// Local Lockout
#define DCL  0x14	// Device Clear
#define PPU  0x15	// Parallel Poll Unconfigure
#define SPE  0x18	// Serial Poll Enable
#define SPD  0x19	// Serial Poll Disable
#define PPE  0x60	// Parallel Poll Enable
#define PPD  0x70	// Parallel Poll Disable

// ibsta mask bits
#define ERR		0x8000	// Error occurs.
#define TIMO	0x4000	// Time-out period elapsed.
#define END		0x2000	// EOI or EOS is detected.
#define SRQI	0x1000	// SRQ is asserted.
#define RQS		0x800	// Device is requesting service.
#define CMPL	0x100	// I/O is completed.
#define LOK		0x80	// Lockout State
#define REM		0x40	// Remote State
#define CIC		0x20	// Controller-in-Charge
#define ATN		0x10	// ATN is asserted.
#define TACS	0x8		// Talker
#define LACS	0x4		// Listener
#define DTAS	0x2		// Device Trigger State
#define DCAS	0x1		// Device Clear State

// error code
#define EDVR	0		// The board or device handle is invalid.
#define ECIC	1		// The interface is not Controller-in-Charge.
#define ENOL	2		// Failed to find Listener.
#define EADR	3		// GPIB address for the device is the same as that for the board, and vice versa.
#define EARG	4		// Invalid parameter is specified.
#define ESAC	5		// The specified board does not have System Controller capability.
#define EABO	6		// Time-out period elapsed.
#define ENEB	7		// The GPIB board is not found.
#define EDMA	8		// Failed to operate DMA transfer.
#define EOIP	10		// Asynchronous I/O is in progress.
#define ECAP	11		// Not supported.
#define EFSO	12		// Failed to access the file.
#define EBUS	14		// Send bus command error.
#define ESTB	15		// Buffer to store the serial poll response byte is overflowed.
#define ESRQ	16		// SRQ remains asserting.
#define ETAB	20		// Insufficient buffer memory

// EOS mode bits
#define BIN		0x1000	// EOS character comparison bit size
#define XEOS	0x800	// Send END with the EOS character.
#define REOS	0x400	// Terminate read on the EOS character.

// Time-out period values
#define TNONE    0		// Infinite
#define T10us    1		// 10 us
#define T30us    2		// 30 us
#define T100us   3		// 100 us
#define T300us   4		// 300 us
#define T1ms     5		// 1 ms
#define T3ms     6		// 3 ms
#define T10ms    7		// 10 ms
#define T30ms    8		// 30 ms
#define T100ms   9		// 100 ms
#define T300ms  10		// 300 ms
#define T1s     11		// 1 s
#define T3s     12		// 3 s
#define T10s    13		// 10 s
#define T30s    14		// 30 s
#define T100s   15		// 100 s
#define T300s   16		// 300 s
#define T1000s  17		// 1000 s

//  ibln Constants
#define NO_SAD   0		// test no secondary address
#define ALL_SAD -1		// test all secondary address


//  ibconfig option values
#define IbcPAD			0x0001	// Primary Address
#define IbcSAD			0x0002	// Secondary Address
#define IbcTMO			0x0003	// Global time-out
#define IbcEOT			0x0004	// Send EOI with last data byte.
#define IbcPPC			0x0005	// Parallel Poll Configure
#define IbcREADDR		0x0006	// Repeat Addressing
#define IbcAUTOPOLL		0x0007	// Auto Serial Polling
#define IbcCICPROT		0x0008	// CIC Protocol
#define IbcIRQ			0x0009	// Use PIO for I/O.
#define IbcSC			0x000A	// System Controller
#define IbcSRE			0x000B	// Assert REN on device calls.
#define IbcEOSrd		0x000C	// Terminate reads on the EOS character.
#define IbcEOSwrt		0x000D	// Send EOI with the EOS character.
#define IbcEOScmp		0x000E	// EOS character comparison size
#define IbcEOSchar		0x000F	// EOS character
#define IbcPP2			0x0010	// Use Parallel Poll Mode 2.
#define IbcTIMING		0x0011	// Bus handshake timing
#define IbcDMA			0x0012	// Use DMA transfer.
#define IbcReadAdjust	0x0013	// Swap bytes on reading.
#define IbcWriteAdjust	0x0014	// Swap bytes on writing.
#define IbcSendLLO		0x0017	// Sending of LLO
#define IbcSPollTime	0x0018	// Serial polls time-out
#define IbcPPollTime	0x0019	// Parallel poll length period
#define IbcEndBitIsNormal 0x001A	// Set END bit of ibsta with EOS character
#define IbcUnAddr		0x001B	// Enable or disable device unaddressing
#define IbcHSCableLength  0x001F	// Cable length for HS488
#define IbcIst			0x0020	// Set the IST bit
#define IbcRsv			0x0021	// Set the RSV byte

// ibask option values
#define IbaPAD			IbcPAD
#define IbaSAD			IbcSAD
#define IbaTMO			IbcTMO
#define IbaEOT			IbcEOT
#define IbaPPC			IbcPPC
#define IbaREADDR		IbcREADDR
#define IbaAUTOPOLL		IbcAUTOPOLL
#define IbaCICPROT		IbcCICPROT
#define IbaIRQ			IbcIRQ
#define IbaSC			IbcSC
#define IbaSRE			IbcSRE
#define IbaEOSrd		IbcEOSrd
#define IbaEOSwrt		IbcEOSwrt
#define IbaEOScmp		IbcEOScmp
#define IbaEOSchar		IbcEOSchar
#define IbaPP2			IbcPP2
#define IbaTIMING		IbcTIMING
#define IbaDMA			IbcDMA
#define IbaReadAdjust	IbcReadAdjust
#define IbaWriteAdjust	IbcWriteAdjust
#define IbaSendLLO		IbcSendLLO
#define IbaSPollTime	IbcSPollTime
#define IbaPPollTime	IbcPPollTime
#define IbaEndBitIsNormal	IbcEndBitIsNormal
#define IbaUnAddr		IbcUnAddr
#define IbaHSCableLength	IbcHSCableLength
#define IbaIst			IbcIst
#define IbaRsv			IbcRsv
#define IbaBNA			0x0200	// A device's access board


// IEEE-488.2 send command option values
#define NULLend			0x00	// Do nothing at the end of a transfer
#define NLend			0x01	// Send NL with EOI after transfer
#define DABend			0x02	// Send EOI with the last DAB

// IEEE-488.2 receive command option value
#define STOPend			0x0100

// Address type (for IEEE-488.2 calls) 
typedef short Addr4882_t;		// System dependent: must be 16 bits 

// Address macro for 488.2 function
#define MakeAddr(pad, sad)	((Addr4882_t)(((pad)&0xFF) | ((sad)<<8)))
#define GetPAD(val)			((val) & 0xFF)
#define GetSAD(val)			(((val) >> 8) & 0xFF)

// Termination code for address list
#ifndef NOADDR
#define NOADDR	(Addr4882_t)((unsigned short)0xFFFF)
#endif

// iblines constants
#define ValidEOI	(short)0x0080
#define ValidATN	(short)0x0040
#define ValidSRQ	(short)0x0020
#define ValidREN	(short)0x0010
#define ValidIFC	(short)0x0008
#define ValidNRFD	(short)0x0004
#define ValidNDAC	(short)0x0002
#define ValidDAV	(short)0x0001
#define BusEOI		(short)0x8000
#define BusATN		(short)0x4000
#define BusSRQ		(short)0x2000
#define BusREN		(short)0x1000
#define BusIFC		(short)0x0800
#define BusNRFD		(short)0x0400
#define BusNDAC		(short)0x0200
#define BusDAV		(short)0x0100

// ibnotify callback function
typedef int (__stdcall * GpibNotifyCallback_t)(int, int, int, long, void *);

#define IBNOTIFY_REARM_FAILED    0xE00A003F


// global variables
extern int  ibsta;
extern int  iberr;
extern int  ibcnt;
extern long ibcntl;

// unicode function
#if defined(UNICODE)
   #define ibbna  ibbnaW
   #define ibfind ibfindW
   #define ibrdf  ibrdfW
   #define ibwrtf ibwrtfW
#else
   #define ibbna  ibbnaA
   #define ibfind ibfindA
   #define ibrdf  ibrdfA
   #define ibwrtf ibwrtfA
#endif

//  IEEE-488.1 Function Prototypes
extern int __stdcall ibfindA   (LPCSTR udname);
extern int __stdcall ibbnaA    (int ud, LPCSTR udname);
extern int __stdcall ibrdfA    (int ud, LPCSTR filename);
extern int __stdcall ibwrtfA   (int ud, LPCSTR filename);

extern int __stdcall ibfindW   (LPCWSTR udname);
extern int __stdcall ibbnaW    (int ud, LPCWSTR udname);
extern int __stdcall ibrdfW    (int ud, LPCWSTR filename);
extern int __stdcall ibwrtfW   (int ud, LPCWSTR filename);

extern int __stdcall ibask    (int ud, int option, PINT v);
extern int __stdcall ibcac    (int ud, int v);
extern int __stdcall ibclr    (int ud);
extern int __stdcall ibcmd    (int ud, PVOID buf, long cnt);
extern int __stdcall ibcmda   (int ud, PVOID buf, long cnt);
extern int __stdcall ibconfig (int ud, int option, int v);
extern int __stdcall ibdev    (int ud, int pad, int sad, int tmo, int eot, int eos);
extern int __stdcall ibdiag   (int ud, PVOID buf, long cnt);
extern int __stdcall ibdma    (int ud, int v);
extern int __stdcall ibeos    (int ud, int v);
extern int __stdcall ibeot    (int ud, int v);
extern int __stdcall ibgts    (int ud, int v);
extern int __stdcall ibist    (int ud, int v);
extern int __stdcall iblines  (int ud, PSHORT result);
extern int __stdcall ibln     (int ud, int pad, int sad, PSHORT listen);
extern int __stdcall ibloc    (int ud);
extern int __stdcall ibnotify (int ud, int mask, GpibNotifyCallback_t Callback, PVOID RefData);
extern int __stdcall ibonl    (int ud, int v);
extern int __stdcall ibpad    (int ud, int v);
extern int __stdcall ibpct    (int ud);
extern int __stdcall ibpoke   (int ud, long option, long v);
extern int __stdcall ibppc    (int ud, int v);
extern int __stdcall ibrd     (int ud, PVOID buf, long cnt);
extern int __stdcall ibrda    (int ud, PVOID buf, long cnt);
extern int __stdcall ibrpp    (int ud, PCHAR ppr);
extern int __stdcall ibrsc    (int ud, int v);
extern int __stdcall ibrsp    (int ud, PCHAR spr);
extern int __stdcall ibrsv    (int ud, int v);
extern int __stdcall ibsad    (int ud, int v);
extern int __stdcall ibsic    (int ud);
extern int __stdcall ibsre    (int ud, int v);
extern int __stdcall ibstop   (int ud);
extern int __stdcall ibtmo    (int ud, int v);
extern int __stdcall ibtrg    (int ud);
extern int __stdcall ibwait   (int ud, int mask);
extern int __stdcall ibwrt    (int ud, PVOID buf, long cnt);
extern int __stdcall ibwrta   (int ud, PVOID buf, long cnt);


// thread-specific global variables
extern int  __stdcall ThreadIbsta (void);
extern int  __stdcall ThreadIberr (void);
extern int  __stdcall ThreadIbcnt (void);
extern long __stdcall ThreadIbcntl (void);


//  IEEE-488.2 Function Prototypes
extern void __stdcall AllSpoll      (int boardID, Addr4882_t * addrlist, PSHORT results);
extern void __stdcall DevClear      (int boardID, Addr4882_t addr);
extern void __stdcall DevClearList  (int boardID, Addr4882_t * addrlist);
extern void __stdcall EnableLocal   (int boardID, Addr4882_t * addrlist);
extern void __stdcall EnableRemote  (int boardID, Addr4882_t * addrlist);
extern void __stdcall FindLstn      (int boardID, Addr4882_t * addrlist, PSHORT results, int limit);
extern void __stdcall FindRQS       (int boardID, Addr4882_t * addrlist, PSHORT dev_stat);
extern void __stdcall PPoll         (int boardID, PSHORT result);
extern void __stdcall PPollConfig   (int boardID, Addr4882_t addr, int dataLine, int lineSense);
extern void __stdcall PPollUnconfig (int boardID, Addr4882_t * addrlist);
extern void __stdcall PassControl   (int boardID, Addr4882_t addr);
extern void __stdcall RcvRespMsg    (int boardID, PVOID buffer, long cnt, int Termination);
extern void __stdcall ReadStatusByte(int boardID, Addr4882_t addr, PSHORT result);
extern void __stdcall Receive       (int boardID, Addr4882_t addr, PVOID buffer, long cnt, int Termination);
extern void __stdcall ReceiveSetup  (int boardID, Addr4882_t addr);
extern void __stdcall ResetSys      (int boardID, Addr4882_t * addrlist);
extern void __stdcall Send          (int boardID, Addr4882_t addr, PVOID databuf, long datacnt, int eotMode);
extern void __stdcall SendCmds      (int boardID, PVOID buffer, long cnt);
extern void __stdcall SendDataBytes (int boardID, PVOID buffer, long cnt, int eot_mode);
extern void __stdcall SendIFC       (int boardID);
extern void __stdcall SendLLO       (int boardID);
extern void __stdcall SendList      (int boardID, Addr4882_t * addrlist, PVOID databuf, long datacnt, int eotMode);
extern void __stdcall SendSetup     (int boardID, Addr4882_t * addrlist);
extern void __stdcall SetRWLS       (int boardID, Addr4882_t * addrlist);
extern void __stdcall TestSRQ       (int boardID, PSHORT result);
extern void __stdcall TestSys       (int boardID, Addr4882_t * addrlist, PSHORT results);
extern void __stdcall Trigger       (int boardID, Addr4882_t addr);
extern void __stdcall TriggerList   (int boardID, Addr4882_t * addrlist);
extern void __stdcall WaitSRQ       (int boardID, PSHORT result);


#ifdef __cplusplus
}
#endif

#endif
